@@ -97,7 +97,7 @@ class ApiAuth(YamlSerializable):
             raise e
         
     @classmethod
-    def save_as_file(cls, auth_filename: str, cred: BrowserAPIAuth | BasicAPIAuth, workspace: str) -> None:
+    def save_as_file(cls, auth_filename: str, cred: ApiAuth, workspace: str) -> None:
         cp = configparser.ConfigParser(default_section=None, interpolation=None)
         if not auth_filename:
             auth_filename = cls.find_user_credentials(auth_filename)
@@ -130,7 +130,7 @@ class ApiAuth(YamlSerializable):
     def factory(conf: dict) -> ApiAuth:
         ex: Exception
 
-        for model_cls in [BasicAPIAuth, BrowserAPIAuth]:
+        for model_cls in [BasicAPIAuth, BrowserAPIAuth, APIKeyAuth]:
             try:
                 return model_cls(**conf)
             except Exception as e:
@@ -160,6 +160,25 @@ class BasicAPIAuth(ApiAuth):
         return {'auth': HTTPBasicAuth(self.user.lower(), self.password)}
 
 
+@dataclass
+class APIKeyAuth(ApiAuth):
+    """
+    Credential model for authenticating to Bigeye using a Personal API Key.
+
+    Attributes:
+        base_url: str - the full url of the bigeye instance.
+        api_key: str - the full API Key provided by Bigeye.
+    """
+    base_url: str
+    """Host URL for the Bigeye -- e.g. app.bigeye.com"""
+    api_key: str
+    """The full API Key provided by Bigeye"""
+
+    def get_auth_headers(self) -> dict:
+        auth_key: str = os.environ.get('BIGEYE_APIKEYAUTH_AUTH_HEADER_KEY', 'Authorization')
+        return {auth_key: f'apikey {self.api_key}'}
+
+
 @dataclass
 class BrowserAPIAuth(ApiAuth):
     """
