@@ -1,10 +1,14 @@
 from __future__ import annotations
 
+import json
 from typing import List, Any
 
 import betterproto
-from pydantic import root_validator
-from pydantic_yaml import YamlModel
+
+import yaml
+from pydantic.v1 import root_validator, BaseModel
+from pydantic_yaml import to_yaml_str
+
 
 from bigeye_sdk.bigconfig_validation.validation_context import register_validation_error
 from bigeye_sdk.bigconfig_validation.validation_functions import safe_split_dict_entry_lines
@@ -12,8 +16,12 @@ from bigeye_sdk.bigconfig_validation.yaml_validation_error_messages import INVAL
     NO_POSSIBLE_MATCH_ERRMSG, POSSIBLE_MATCH_ERRMSG, FORMATTING_ERRMSG, UNKNOWN_SERIALIZATION_ERROR
 from bigeye_sdk.functions.search_and_match_functions import fuzzy_match
 
+from bigeye_sdk.log import get_logger
 
-class YamlModelWithValidatorContext(YamlModel):
+log = get_logger(__file__)
+
+
+class YamlModelWithValidatorContext(BaseModel):
     @classmethod
     def register_validation_error(cls, error_lines: List[str], error_message: str,
                                   error_context_lines: List[str] = None):
@@ -42,8 +50,9 @@ class YamlModelWithValidatorContext(YamlModel):
         Returns: List of yaml string lines.
 
         """
-        return self.yaml(exclude_unset=True, exclude_defaults=True,
-                         exclude_none=True, indent=True).splitlines()
+        data = self.json(exclude_unset=True, exclude_defaults=True,
+                         exclude_none=True, indent=True)
+        return yaml.safe_dump(json.loads(data)).splitlines()
 
     @root_validator(pre=True)
     def verify_all_attributes_valid(cls, values):
