@@ -1,21 +1,23 @@
 from __future__ import annotations
 
+import json
 import time
 from abc import ABC
 from json import JSONDecodeError
 from typing import List, Optional, Dict, Tuple
+from typing_extensions import deprecated
 
 import requests
 
-from bigeye_sdk.authentication.api_authentication import BasicAPIAuth, BrowserAPIAuth, ApiAuth
+from bigeye_sdk.authentication.api_authentication import BasicAPIAuth, BrowserAPIAuth, ApiAuth, APIKeyAuth
 from bigeye_sdk.bigconfig_validation.big_config_reports import MetricSuiteReport, ProcessStage
 from bigeye_sdk.client.base_client import BaseApiClient
 from bigeye_sdk.client.enum import Method
 from bigeye_sdk.authentication.config import WorkspaceConfig
 from bigeye_sdk.client.generated_datawatch_client import GeneratedDatawatchClient
 from bigeye_sdk.exceptions.exceptions import NoSourcesFoundException, TableNotFoundException, \
-    BigconfigIncompleteException, WorkspaceNotSetException
-from bigeye_sdk.functions.delta_functions import infer_column_mappings, build_ccm, match_tables_by_name
+    BigconfigIncompleteException, WorkspaceNotSetException, AuthenticationFailedException
+from bigeye_sdk.functions.delta_functions import infer_column_mappings, build_ccm
 from bigeye_sdk.functions.metric_functions import set_default_model_type_for_threshold, is_freshness_metric
 from bigeye_sdk.functions.table_functions import get_table_column_priority_first, table_has_metric_time, \
     fully_qualified_table_to_elements
@@ -54,8 +56,13 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     UpsertDeltaResponse,
     OwnableType,
     SetObjectOwnerRequest,
+    TableLineageV2Response,
     ObjectOwnerResponse, SendDbtCoreRunInfoResponse, SendDbtCoreRunInfoRequest, CustomRuleBulkRequest, BulkResponse,
-    CustomRulesThresholdType, MetricSchedule, CreateCustomRuleRequest, CustomRule, CustomRuleInfo,
+    CustomRulesThresholdType, MetricSchedule, CustomRuleInfo,
+    MetricTemplateParameterType,
+    GetCustomRuleListResponse, Workspace, User, SearchType, SearchResponse, SearchRequest, DataNodeType,
+    LineageSearchResponse, LineageSearchRequest,
+    GetCustomRuleListResponse, Workspace, User, BigconfigWorkflowV2StatusResponse
 )
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.delta_facade import SimpleDeltaConfiguration
@@ -63,12 +70,11 @@ from bigeye_sdk.model.metric_facade import SimpleUpsertMetricRequest
 from bigeye_sdk.model.protobuf_enum_facade import SimpleCatalogEntityType
 from bigeye_sdk.model.protobuf_extensions import MetricDebugQueries
 
-
 log = get_logger(__file__)
 
 
 def datawatch_client_factory(auth: ApiAuth, workspace_config: WorkspaceConfig = None, workspace_id: int = None):
-    if isinstance(auth, (BasicAPIAuth, BrowserAPIAuth)):
+    if isinstance(auth, (BasicAPIAuth, BrowserAPIAuth, APIKeyAuth)):
         if workspace_config:
             client = DatawatchClient(auth=auth, workspace_config=workspace_config)
         elif workspace_id:
@@ -112,18 +118,67 @@ def get_user_auth(auth: ApiAuth) -> UserAuth:
             url=f'{auth.base_url}{url}',
             cookies=auth.auth_factory().get_cookies(),
             headers=headers)
+    # API key not supported for /auth or /ajaxsignin
+    elif isinstance(auth, APIKeyAuth):
+        raise WorkspaceNotSetException("Workspace ID is required when using API key authentication.")
+
+    if response.status_code < 200 or response.status_code >= 300:
+        log.error("Error trying to retrieve workspaces.")
+        if response.status_code == 401:
+            raise AuthenticationFailedException(
+                "Authentication failed. Verify credentials or that you are logged in to Bigeye via a supported browser."
+            )
+        raise Exception(f'Error returned from datawatch: {response.status_code} - {response.reason}')
 
     return UserAuth().from_dict(response.json())
 
 
+def __get_all_workspace_client(cred: ApiAuth) -> DatawatchClient:
+    return datawatch_client_factory(
+        auth=cred, workspace_config=WorkspaceConfig(workspace_id=0)
+    )
+
+
+def create_api_key_for_configuration(cred: ApiAuth, name: str, description: Optional[str] = "") -> str:
+    return __get_all_workspace_client(cred=cred).create_personal_api_key(name=name, description=description).api_key
+
+
+def create_agent_api_key(cred: ApiAuth, name: str, description: Optional[str] = "") -> str:
+    return __get_all_workspace_client(cred=cred).create_agent_api_key(name=name, description=description).api_key
+
+
+def get_all_workspaces_for_login(cred: ApiAuth) -> List[Workspace]:
+    return __get_all_workspace_client(cred=cred).get_workspaces().workspaces
+
+
+def get_all_users_for_login(cred: ApiAuth) -> List[User]:
+    return __get_all_workspace_client(cred=cred).get_users().users
+
+
+def get_allowed_emails_for_workspace(cred: ApiAuth, workspace_id: int) -> List[str]:
+    return [user.email.lower() for user in
+            __get_all_workspace_client(cred=cred).get_workspace_accessors(workspace_id=workspace_id).users
+            ]
+
+def verify_agent_api_key(self, *, base_url: str, agent_api_key: str) -> bool:
+    response = requests.get(url=f"{base_url}/api/v1/agent-api-keys/verify",
+                            headers={"Authorization": f"apikey {agent_api_key}"})
+    return True if response.status_code == 204 else False
+
+def verify_personal_api_key(self, *, base_url: str, personal_api_key: str) -> bool:
+    response = requests.get(url=f"{base_url}/api/v1/personal-api-keys/verify",
+                            headers={"Authorization": f"apikey {personal_api_key}"})
+    return True if response.status_code == 204 else False
+
+
 class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
-    def __init__(self, auth: BasicAPIAuth | BrowserAPIAuth, workspace_config: WorkspaceConfig):
+    def __init__(self, auth: BasicAPIAuth | BrowserAPIAuth | APIKeyAuth, workspace_config: WorkspaceConfig):
         super().__init__(base_url=auth.base_url)
         self._auth = auth
         self.config = workspace_config
 
-    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None):
+    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None, timeout: int = None, proxies: dict = {}):
         try:
             fq_url = f'{self._base_url}{url}'
             log.info(f'Request Type: {method.name}; URL: {fq_url}; Body: {body}')
@@ -135,10 +190,15 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
                 'url': fq_url,
                 'headers': headers,
                 'data': body,
-                'params': params
+                'params': params,
+                'timeout': timeout,
+                'proxies': proxies
             }
 
-            kwargs.update(self._auth.get_auth_headers())
+            if isinstance(self._auth, APIKeyAuth):
+                headers.update(self._auth.get_auth_headers())
+            else:
+                kwargs.update(self._auth.get_auth_headers())
 
             if method == Method.GET:
                 response = requests.get(**kwargs)
@@ -156,6 +216,10 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
         else:
             log.info(f'Return Code: {response.status_code}')
             if response.status_code < 200 or response.status_code >= 300:
+                if response.status_code == 401:
+                    raise AuthenticationFailedException(
+                        "API returned 401 unauthorized. Verify the auth method being used is correct."
+                    )
                 log.error(f'Error code returned from datawatch: {response.status_code} - {response.reason}')
                 raise Exception(response.text)
             else:
@@ -179,11 +243,11 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             self.delete_metric(m)
 
     def get_table_level_metrics(self) -> MetricNamesResponse:
+        return MetricNamesResponse().from_dict(self.get_table_level_metrics_raw())
+
+    def get_table_level_metrics_raw(self) -> dict:
         url = '/api/v1/metrics/table-level-metric-names'
-
-        response = self._call_datawatch(Method.GET, url=url)
-
-        return MetricNamesResponse().from_dict(response)
+        return self._call_datawatch(Method.GET, url=url)
 
     def get_sources_by_name(self, source_names: List[str] = None) -> Dict[str, Source]:
         """
@@ -403,10 +467,10 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
             source_ids = [s.id for s in self.get_sources_by_name(source_names=warehouses).values()]
 
-            tables_ix = {t.id: t for t in self.get_tables(ids=table_ids+target_ids,
-                                                            schema=schemas,
-                                                            table_name=table_names+target_names,
-                                                            warehouse_id=source_ids).tables}
+            tables_ix = {t.id: t for t in self.get_tables(ids=table_ids + target_ids,
+                                                          schema=schemas,
+                                                          table_name=table_names + target_names,
+                                                          warehouse_id=source_ids).tables}
 
             tables = list(tables_ix.values())
 
@@ -443,7 +507,7 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             for target_compare in target_comparisons.values():
                 target_table = tables_ix[target_compare.target_table_id]
                 response_as_ctc = ComparisonTableConfiguration(name=sdc.delta_name, source_table_id=source_table.id,
-                                                                target_table_id=target_compare.target_table_id)
+                                                               target_table_id=target_compare.target_table_id)
 
                 if target_compare.delta_column_mapping:
                     """
@@ -485,7 +549,8 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
                         break
                     else:
                         try:
-                            schedule = self.create_named_schedule(name=sdc.cron_schedule.name, cron=sdc.cron_schedule.cron)
+                            schedule = self.create_named_schedule(name=sdc.cron_schedule.name,
+                                                                  cron=sdc.cron_schedule.cron)
                             named_schedule = IdAndDisplayName(id=schedule.id, display_name=schedule.name)
                             break
                         except:
@@ -493,7 +558,6 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
                             log.info("The delta will be created without a schedule.")
                             break
 
-
             delta = Delta(
                 name=sdc.delta_name,
                 source_table=IdAndDisplayName(id=source_table.id, display_name=source_table.name),
@@ -585,7 +649,6 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         return CreateComparisonTableResponse().from_dict(response)
 
-
     def upsert_delta(self, delta: Delta):
 
         url = "/api/v1/deltas"
@@ -593,14 +656,19 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
         response = self._call_datawatch(Method.POST, url, request.to_json())
         return UpsertDeltaResponse().from_dict(response)
 
-    def purge_metric_suites(self, source_names: List[str], purge_all_sources: bool = False,
-                            apply: bool = False) -> List[MetricSuiteReport]:
+    def purge_metric_suites(self,
+                            source_names: List[str],
+                            purge_all_sources: bool = False,
+                            apply: bool = False,
+                            namespace: Optional[str] = None
+    ) -> List[MetricSuiteReport]:
         """
         Purges metric suites for all warehouse_ids.
         Args:
             source_names: one or more source names
             purge_all_sources: If true will purge all sources in workspace.
             apply: whether to apply the change
+            namespace: the namespace to purge
 
         return: list of metric suite responses.
 
@@ -620,24 +688,26 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         reports: List[MetricSuiteReport] = []
 
-        process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
-
         if source_name_id_tuples:
+            process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
             for source_id, source_name in source_name_id_tuples:
-                """Creates and posts an empty metric suite.  backend will, by default, remove all metrics based on an 
-                empty metric metric suite."""
-                metric_suite = MetricSuite(source_id=source_id)
-                response = self.post_metric_suite(metric_suite=metric_suite, apply=apply)
+                metric_suite = MetricSuite(source_id=source_id, namespace=namespace)
+                response = self.post_bigconfig(metric_suite=metric_suite, apply=apply)
                 reports.append(MetricSuiteReport.from_datawatch_object(obj=response,
                                                                        source_name=source_name,
-                                                                       process_stage=process_stage)
-                               )
+                                                                       process_stage=process_stage))
 
             return reports
 
         else:
             raise NoSourcesFoundException('No sources identified for purge.')
 
+    @deprecated("Use post_bigconfig with an empty MetricSuite for the source")
+    def purge_warehouse(self, source_id: int) -> BulkResponse:
+        url = f"/api/v1/metric-suites/purge/warehouse/{source_id}"
+        return BulkResponse().from_dict(self._call_datawatch(Method.GET, url=url))
+
+    @deprecated("Use post_bigconfig")
     def post_metric_suite(self, metric_suite: MetricSuite, apply: bool = False) -> MetricSuiteResponse:
         mc_url = "/api/v1/metric-suites"
         if apply:
@@ -649,13 +719,13 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             self._call_datawatch(Method.POST, url=url, body=metric_suite.to_json())
         )
 
+    @deprecated("Use post_bigconfig")
     def post_metric_suite_queue(self, metric_suite: MetricSuite, apply: bool = False) -> MetricSuiteResponse:
         if apply:
             url = "/api/v1/metric-suites/queue"
         else:
             url = f"/api/v1/metric-suites/dry-run/queue"
 
-        # TODO update once backend returns a BigconfigWorkflowStatusResponse for inital call to queue
         response: WorkflowStatusResponse = WorkflowStatusResponse().from_dict(
             self._call_datawatch(Method.POST, url=url, body=metric_suite.to_json())
         )
@@ -677,6 +747,34 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         return bwsr.metric_suite_response
 
+    def post_bigconfig(self, metric_suite: MetricSuite, apply: bool = False) -> MetricSuiteResponse:
+        bc_url = "/api/v1/bigconfig"
+        if apply:
+            url = f"{bc_url}/apply"
+        else:
+            url = f"{bc_url}/plan"
+
+        response: BigconfigWorkflowV2StatusResponse = BigconfigWorkflowV2StatusResponse().from_dict(
+            self._call_datawatch(Method.POST, url=url, body=metric_suite.to_json())
+        )
+
+        status_url = f"{bc_url}/status/{response.status_response.workflow_v2_id.workflow_id}"
+        while response.status_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED or \
+                response.status_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS:
+            log.info(f"Bigconfig in queue...")
+            time.sleep(10)
+            response: BigconfigWorkflowV2StatusResponse = BigconfigWorkflowV2StatusResponse().from_dict(
+                self._call_datawatch(Method.GET, url=status_url)
+            )
+
+        log.info(f"Queuing complete. Workflow finished with status {response.status_response.status.name}")
+
+        if response.status_response.status != WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_COMPLETED:
+            err = f"Bigconfig workflow was not completed. Final status: {response.status_response.status.name}"
+            raise BigconfigIncompleteException(err)
+
+        return response.bigconfig_response
+
     def upsert_metric(
             self,
             *,
@@ -733,7 +831,18 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
 
         url = "/api/v1/metrics"
 
-        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        # this is done so that default 0 values are not removed when converting to json
+        request_dict = request.to_dict()
+        if request_dict["thresholds"][0] == {}:
+            request_dict["thresholds"] = [{"noneThreshold": {}}]
+
+        # if both schedule and name schedule are provided the named schedule should be used.
+        if request_dict.get("scheduleFrequency", None) and request_dict.get("metricSchedule", None):
+            request_dict.pop('scheduleFrequency', None)
+
+        request_json = json.dumps(request_dict)
+
+        response = self._call_datawatch(Method.POST, url=url, body=request_json)
 
         return MetricConfiguration().from_dict(response)
 
@@ -810,19 +919,21 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
             url = f"/statistics/backfillAutoThresholds/{metric_id}"
             response = self._call_datawatch(Method.GET, url=url)
 
-    def get_debug_queries(self, *, metric_ids: List[int]) -> List[MetricDebugQueries]:
+    def get_debug_queries(self, *, metric_ids: List[int], timeout: int = None) -> List[MetricDebugQueries]:
         """
         Get queries for debug page
         :param metric_ids: List of metric ids for which to retrieve debug queries.
+        :param timeout: The time in seconds to wait for a response from Bigeye.
         :return: a dictionary of
         """
         r: List[MetricDebugQueries] = []
 
         for metric_id in metric_ids:
             url = f'/api/v1/metrics/{metric_id}/debug/queries'
+            response = self._call_datawatch(Method.GET, url, timeout=timeout)
             i = MetricDebugQueries(
                 metric_id=metric_id,
-                debug_queries=GetDebugQueriesResponse().from_dict(self._call_datawatch(Method.GET, url))
+                debug_queries=GetDebugQueriesResponse().from_dict(response)
             )
             r.append(i)
 
@@ -885,7 +996,6 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
                                job_url: Optional[str] = None,
                                job_run_url: Optional[str] = None) -> SendDbtCoreRunInfoResponse:
 
-
         url = "/api/v1/integrations/dbt/job-run"
         request = SendDbtCoreRunInfoRequest()
         request.project_name = project_name
@@ -963,3 +1073,51 @@ class DatawatchClient(BaseApiClient, GeneratedDatawatchClient, ABC):
                 custom_rule=custom_rule,
                 rule_id=rule_id
             )
+
+    def get_rules(self, warehouse_id: int = None, collection_id: int = None) -> GetCustomRuleListResponse:
+        """Get all rules for a given warehouse or collection. Either the warehouse ID or collection ID is required."""
+        if not warehouse_id and not collection_id:
+            raise Exception(f"Either the warehouse ID or collection ID are required.")
+        elif warehouse_id and not collection_id:
+            return self.get_rules_for_source(warehouse_id=warehouse_id)
+        elif collection_id and not warehouse_id:
+            return self.get_rules_for_collection(collection_id=collection_id)
+        elif warehouse_id and collection_id:
+            rules = self.get_rules_for_source(warehouse_id=warehouse_id)
+            rules_filtered = []
+            for r in rules.custom_rules:
+                for c in r.collections:
+                    if c.id == collection_id:
+                        rules_filtered.append(r)
+            rules.custom_rules = rules_filtered
+            return rules
+
+    def get_upstream_nodes(self, node_id: int, depth: Optional[int] = None) -> TableLineageV2Response:
+        url = f"/api/v2/lineage/nodes/{node_id}/graph?direction=upstream"
+        if depth:
+            url += f"&depth={depth}"
+        return TableLineageV2Response().from_dict(
+            self._call_datawatch_impl(method=Method.GET, url=url)
+        )
+
+    def get_downstream_nodes(self, node_id: int, depth: Optional[int] = None) -> TableLineageV2Response:
+        url = f"/api/v2/lineage/nodes/{node_id}/graph?direction=downstream"
+        if depth:
+            url += f"&depth={depth}"
+        response = self._call_datawatch_impl(method=Method.GET, url=url)
+        return TableLineageV2Response(**response)
+
+    def search_lineage(self, search: str, search_type: Optional[DataNodeType] = None, limit: Optional[int] = 100) -> SearchResponse:
+        url = f"/api/v1/search"
+        request = SearchRequest()
+        request.search = search
+        request.limit = limit
+
+        # properly set the search types
+        request_dict = request.to_dict()
+        if search_type is not None:
+            search_types_dict = {"dataNodeType": search_type.name}
+            request_dict["types"] = [search_types_dict]
+
+        response = self._call_datawatch(Method.POST, url=url, body=json.dumps(request_dict))
+        return SearchResponse().from_dict(response)
