@@ -7,13 +7,11 @@ from time import sleep, time
 from typing import List, Optional, Union, Dict
 
 from bigeye_sdk.client.enum import Method
-from bigeye_sdk.functions.metric_functions import is_same_metric, is_same_column_metric
+from bigeye_sdk.functions.metric_functions import is_same_metric, is_same_column_metric, create_metric_info_list
 from bigeye_sdk.functions.urlfuncts import encode_url_params
 from bigeye_sdk.generated.com.bigeye.models.generated import (
     TableList,
     ColumnMetricProfileList,
-    AutometricAcceptRequest,
-    AutometricAcceptResponse,
     GetMetricInfoListRequest,
     MetricInfoList,
     SearchMetricConfigurationRequest,
@@ -88,7 +86,6 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     GetSchemaChangesResponse,
     SchemaChange,
     IntegrationsResponse,
-    TableauWorkbook,
     Integration,
     GetIntegrationEntitiesResponse,
     ConfigValue,
@@ -109,14 +106,12 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     Group,
     RoleV2ListResponse,
     RoleOperation,
-    BulkChangeGroupRolesRequest,
     BulkResponse,
     GetVirtualTableListRequest,
     GetVirtualTableListResponse,
     IssueMessageUpdate,
     IssueConfigUpdate,
     IssueAssignmentUpdate,
-    IssueSortField,
     WorkflowInfo,
     WorkflowProcessingStatus,
     RunDeltaResponse,
@@ -130,14 +125,22 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     GetColumnListResponse,
     MetricInfo, GetTableListResponse, GetTableListRequest, GetCustomRuleListResponse,
     CreateCustomRuleRequest, CustomRule, MetricSchedule, CustomRulesThresholdType, UpdateCustomRuleRequest,
-    CustomRuleInfo, TimeInterval, TimeIntervalType
+    CustomRuleInfo, TimeInterval, TimeIntervalType, RebuildSourceRequest, CreatePersonalApiKeyRequest,
+    CreatePersonalApiKeyResponse, ListPersonalApiKeyResponse, CreateAgentApiKeyResponse, CreateAgentApiKeyRequest,
+    ListAgentApiKeyResponse, GetWorkspaceAccessorsResponse, CreateLineageNodeV2Request, LineageNodeV2,
+    LineageSearchResponse, LineageSearchRequest, GetDebugQueriesResponse, ConfigValueType, SourceMetadataOverrides,
+    WarehouseType, BulkChangeGroupGrantsRequest, Grant, RoleV2, IdAndDisplayName, BulkChangeGroupGrantsResponse,
+    IssuePriorityChangeEvent, TableLineageV2Response, CreateLineageNodeV2BulkRequest, CreateLineageEdgeV2BulkRequest,
+    GetMetricObservedColumnBulkRequest, MetricObservedColumnListResponse, MetricObservedColumnRequest,
+    MetricObservedColumnResponse, GetCustomRuleListRequest
 )
 
-
 # create logger
 from bigeye_sdk.log import get_logger
+from bigeye_sdk.model.enums import LineageDirection
 from bigeye_sdk.model.protobuf_enum_facade import SimpleFieldType, SimpleMetricTemplateParameterType, \
- SimpleWorkflowProcessingStatus, SimpleTableSortField, SimpleSortDirection
+    SimpleWorkflowProcessingStatus, SimpleTableSortField, SimpleSortDirection, SimpleIssueStatus, SimpleIssuePriority, \
+    SimpleIssueSortField
 
 log = get_logger(__file__)
 
@@ -145,12 +148,16 @@ log = get_logger(__file__)
 class GeneratedDatawatchClient(abc.ABC):
     """TODO: In future, should be generated and only contain methods generated from protobuf. :)"""
 
-    def _call_datawatch(self, method: Method, url, body: str = None, params: dict = None):
+    def _call_datawatch(
+            self, method: Method, url, body: str = None, params: dict = None, timeout: int = None, proxies: dict = {}
+    ):
         url = url.replace('//', '/')
-        return self._call_datawatch_impl(method=method, url=url, body=body, params=params)
+        return self._call_datawatch_impl(
+            method=method, url=url, body=body, params=params, timeout=timeout, proxies=proxies
+        )
 
     @abc.abstractmethod
-    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None):
+    def _call_datawatch_impl(self, method: Method, url, body: str = None, params: dict = None, timeout: int = None, proxies: dict = {}):
         """Each implementation must override this."""
         pass
 
@@ -178,6 +185,7 @@ class GeneratedDatawatchClient(abc.ABC):
                       schema_id: List[int] = [],
                       include_favorites: bool = False,
                       ignore_fields: bool = False,
+                      include_data_node_ids: bool = True
                       ) -> TableList:
         url = f"/api/v1/tables{encode_url_params(locals(), remove_keys=['api_conf'])}"
         log.info('Getting warehouse tables.')
@@ -197,7 +205,8 @@ class GeneratedDatawatchClient(abc.ABC):
                         sort_field: SimpleTableSortField = SimpleTableSortField.UNSPECIFIED,
                         sort_direction: SimpleSortDirection = SimpleSortDirection.UNSPECIFIED,
                         search: str = "",
-                        ignore_fields: bool = True) -> GetTableListResponse:
+                        ignore_fields: bool = True,
+                        include_data_node_ids: bool = False) -> GetTableListResponse:
 
         url = "/api/v1/tables/fetch"
         request = GetTableListRequest()
@@ -210,8 +219,13 @@ class GeneratedDatawatchClient(abc.ABC):
         request.sort_direction = sort_direction.to_datawatch_object()
         request.search = search
         request.ignore_fields = ignore_fields
+        request.include_data_node_ids = include_data_node_ids
 
-        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        request_dict = request.to_dict()
+        request_dict["ignoreFields"] = ignore_fields
+        request_json = json.dumps(request_dict)
+
+        response = self._call_datawatch(Method.POST, url=url, body=request_json)
 
         tlist_current = GetTableListResponse().from_dict(response)
         tlist_total = GetTableListResponse()
@@ -219,14 +233,16 @@ class GeneratedDatawatchClient(abc.ABC):
 
         while tlist_current.pagination_info.next_cursor:
             request.page_cursor = tlist_current.pagination_info.next_cursor
-            response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+            request_dict = request.to_dict()
+            request_dict["ignoreFields"] = ignore_fields
+            request_json = json.dumps(request_dict)
+
+            response = self._call_datawatch(Method.POST, url=url, body=request_json)
             tlist_current = GetTableListResponse().from_dict(response)
             tlist_total.tables.extend(tlist_current.tables)
 
         return tlist_total
 
-
-
     def get_table_ids(self,
                       warehouse_id: List[int] = [],
                       schemas: List[str] = [],
@@ -264,21 +280,6 @@ class GeneratedDatawatchClient(abc.ABC):
         url = f'/statistics'
         return self._call_datawatch(Method.DELETE, url, body)
 
-    def accept_autometrics(
-            self, *, metric_ids: List[int] = [], lookback_days: int = 7
-    ) -> AutometricAcceptResponse:
-        """Batch accept autometrics"""
-
-        request = AutometricAcceptRequest()
-        request.metric_ids = metric_ids
-        request.lookback_days = lookback_days
-
-        url = f'/api/v1/metrics/autometrics/accept'
-        body = request.to_json()
-        log.info(f'Deploying autometrics: {url} -- {body}')
-        r = self._call_datawatch(Method.POST, url, body)
-        return AutometricAcceptResponse().from_dict(r)
-
     def get_metric_configuration(
             self, *, metric_id: int = 0
     ) -> MetricConfiguration:
@@ -378,14 +379,23 @@ class GeneratedDatawatchClient(abc.ABC):
         else:
             response = self._call_datawatch(Method.GET, url=url)
 
-        mil_current = MetricInfoList().from_dict(response)
+        try:
+            mil_current = MetricInfoList().from_dict(response)
+        except ValueError as e:
+            log.warning(f"Exception handled during response {e}")
+            log.info("Some metrics may not be available")
+            mil_current = create_metric_info_list(metric_infos=response)
+
         mil_return = MetricInfoList()
         mil_return.metrics.extend(mil_current.metrics)
 
         while mil_current.pagination_info.next_cursor:
             request.page_cursor = mil_current.pagination_info.next_cursor
             response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
-            mil_current = MetricInfoList().from_dict(response)
+            try:
+                mil_current = MetricInfoList().from_dict(response)
+            except ValueError:
+                mil_current = create_metric_info_list(metric_infos=response)
             mil_return.metrics.extend(mil_current.metrics)
 
         return mil_return
@@ -558,7 +568,7 @@ class GeneratedDatawatchClient(abc.ABC):
 
         Returns:  Optional[Delta]
         """
-    
+
         url = f"/api/v1/deltas/run/{delta_id}"
         response = RunDeltaResponse().from_dict(
             self._call_datawatch(Method.GET, url)
@@ -571,17 +581,17 @@ class GeneratedDatawatchClient(abc.ABC):
         workflow_response = self.get_workflow_status(
             workflow_id=response.workflow_info.workflow_id
         )
-    
+
         while (
-            workflow_response.status
-            == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED
-            or workflow_response.status
-            == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS
+                workflow_response.status
+                == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED
+                or workflow_response.status
+                == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS
         ):
             log.info("Delta run in queue...")
             sleep(10)
             workflow_response = self.get_workflow_status(workflow_id=workflow_response.workflow_id)
-    
+
         status = SimpleWorkflowProcessingStatus.from_datawatch_object(
             workflow_response.status
         )
@@ -627,20 +637,25 @@ class GeneratedDatawatchClient(abc.ABC):
 
         return GetComparisonTableInfosResponse().from_dict(response).comparison_table_infos
 
-    def get_deltas(self, *, comparison_table_ids: List[int] = None,
-                            exclude_comparison_metrics: bool = True) -> List[DeltaInfo]:
+    def get_deltas(self,
+                   *,
+                   comparison_table_ids: List[int] = [],
+                   exclude_comparison_metrics: bool = True,
+                   page_size: Optional[int] = 0) -> List[DeltaInfo]:
         """
             Args:
                 comparison_table_ids: Optional.  The id or IDs of tables used in deltas.
                 exclude_comparison_metrics: Optional. Whether to exclude the list of ComparisonMetricInfos
+                page_size: Optional. The number of records to return per search
 
-            Returns:  List[ComparisonTableInfo]
+            Returns:  List[DeltaInfo]
         """
 
         url = "/api/v1/deltas/fetch"
         request = GetComparisonTableInfosRequest()
         request.comparison_table_ids = comparison_table_ids
         request.exclude_comparison_metrics = exclude_comparison_metrics
+        request.page_size = page_size
 
         response = self._call_datawatch(Method.POST, url, request.to_json())
 
@@ -687,7 +702,7 @@ class GeneratedDatawatchClient(abc.ABC):
         return BatchRunMetricsResponse().from_dict(response)
 
     def run_metric_batch_async(
-            self, *, metric_ids: List[int] =[]
+            self, *, metric_ids: List[int] = []
     ) -> List[MetricInfo]:
         """Batch run metrics via queue"""
         url = "/api/v1/metrics/run/batch/queue"
@@ -701,7 +716,7 @@ class GeneratedDatawatchClient(abc.ABC):
         workflow_response = self.get_workflow_status(workflow_id=response.workflow_id)
 
         while (workflow_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_QUEUED or
-                workflow_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS):
+               workflow_response.status == WorkflowProcessingStatus.WORKFLOW_PROCESSING_STATUS_IN_PROGRESS):
             log.info("Batch metric run in queue...")
             sleep(10)
             workflow_response = self.get_workflow_status(workflow_id=response.workflow_id)
@@ -782,6 +797,60 @@ class GeneratedDatawatchClient(abc.ABC):
         log.info(f'Source {source.name} created with warehouse ID: {source.id}')
         return source
 
+    def upsert_source(
+            self,
+            *,
+            source_id: int = 0,
+            name: str = "",
+            hostname: str = "",
+            port: int = 0,
+            database_name: str = "",
+            database_type: WarehouseType = WarehouseType.DATABASE_TYPE_UNSPECIFIED,
+            domain: str = "",
+            username: str = "",
+            password: str = "",
+            private_key_file: str = "",
+            query_timeout_seconds: int = 0,
+            skip_indexing: bool = False,
+            alation_source_id: int = 0,
+            atlan_connection_id: str = "",
+            temporal_agent_secret: str = "",
+            bigquery_query_project_ids: str = "",
+            source_metadata_overrides: SourceMetadataOverrides = SourceMetadataOverrides(),
+            request: Optional[CreateSourceRequest] = None
+    ):
+
+        url = '/api/v1/sources'
+        if not request:
+            request = CreateSourceRequest()
+            request.id = source_id
+            request.name = name
+            request.hostname = hostname
+            request.port = port
+            request.database_name = database_name
+            request.database_type = database_type
+            request.domain = domain
+            request.username = username
+            request.password = password
+            request.private_key_file = private_key_file
+            request.query_timeout_seconds = query_timeout_seconds
+            request.skip_indexing = skip_indexing
+            request.alation_source_id = alation_source_id
+            request.atlan_connection_id = atlan_connection_id
+            request.temporal_agent_secret = temporal_agent_secret
+            request.bigquery_query_project_ids = bigquery_query_project_ids
+            request.source_metadata_overrides = source_metadata_overrides
+
+        response = self._call_datawatch(Method.POST, url, request.to_json())
+        source = Source().from_dict(value=response['source'])
+
+        if source_id:
+            log.info(f"Source {source_id} has been updated.")
+        else:
+            log.info(f"Source {source.name} created with warehouse ID: {source.id}")
+        return source
+
+
     def delete_metric(self, metric: Union[int, MetricConfiguration]):
         """
         Deletes an individual metric.
@@ -812,7 +881,7 @@ class GeneratedDatawatchClient(abc.ABC):
     def get_issues(
             self,
             *,
-            current_status: List[IssueStatus] = [],
+            current_status: List[SimpleIssueStatus] = [],
             warehouse_ids: List[int] = [],
             schema_ids: List[int] = [],
             metric_ids: List[int] = [],
@@ -822,15 +891,20 @@ class GeneratedDatawatchClient(abc.ABC):
             table_ids: List[int] = [],
             column_ids: List[int] = [],
             related_issue_ids: List[int] = [],
-            sort_field: IssueSortField = IssueSortField.ISSUE_SORT_FIELD_UNSPECIFIED,
-            sort_direction: SortDirection = SortDirection.SORT_DIRECTION_UNSPECIFIED) -> List[Issue]:
+            parent_issue_ids: List[int] = [],
+            priority: List[SimpleIssuePriority] = [],
+            assignee_ids: List[int] = [],
+            page_size: int = 0,
+            sort_field: SimpleIssueSortField = SimpleIssueSortField.UNSPECIFIED,
+            sort_direction: SimpleSortDirection = SimpleSortDirection.UNSPECIFIED,
+            search: str = "") -> List[Issue]:
 
         url = '/api/v1/issues/fetch'
 
         # TODO: Will require update when GetIssuesRequest refactors to collection_ids
 
         request = GetIssuesRequest()
-        request.current_status = current_status
+        request.current_status = [status.to_datawatch_object() for status in current_status]
         request.warehouse_ids = warehouse_ids
         request.schema_ids = schema_ids
         request.metric_ids = metric_ids
@@ -840,8 +914,13 @@ class GeneratedDatawatchClient(abc.ABC):
         request.table_ids = table_ids
         request.column_ids = column_ids
         request.related_issue_ids = related_issue_ids
-        request.sort_field = sort_field
-        request.sort_direction = sort_direction
+        request.parent_issue_ids = parent_issue_ids
+        request.sort_field = sort_field.to_datawatch_object()
+        request.sort_direction = sort_direction.to_datawatch_object()
+        request.page_size = page_size
+        request.priority = [p.to_datawatch_object() for p in priority]
+        request.assignee_ids = assignee_ids
+        request.search = search
 
         response = GetIssuesResponse().from_dict(
             self._call_datawatch(method=Method.POST, url=url, body=request.to_json())
@@ -849,10 +928,10 @@ class GeneratedDatawatchClient(abc.ABC):
         issues: List[Issue] = response.issue
         while response.pagination_info.next_cursor:
             request.page_cursor = response.pagination_info.next_cursor
-            response = self._call_datawatch(method=Method.POST, url=url, body=request.to_json())
-            issues.extend(
-                GetIssuesResponse().from_dict(response).issue
+            response = GetIssuesResponse().from_dict(
+                self._call_datawatch(method=Method.POST, url=url, body=request.to_json())
             )
+            issues.extend(response.issue)
         return issues
 
     def get_issue(self, issue_id: int) -> Issue:
@@ -921,6 +1000,13 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.PUT, url, request.to_json())
         return UpdateIssueResponse().from_dict(response).issue
 
+    def update_issue_priority(self, issue_id: int, priority: SimpleIssuePriority) -> Issue:
+        url = f'/api/v1/issues/{issue_id}'
+        priority_update = IssuePriorityChangeEvent(issue_priority=priority.to_datawatch_object())
+        request = UpdateIssueRequest(priority_update=priority_update)
+        response = self._call_datawatch(Method.PUT, url, request.to_json())
+        return UpdateIssueResponse().from_dict(response).issue
+
     def get_debug_preview(self, *, metric_id: int = 0) -> GetPreviewResponse:
         """Get sample debug rows for debug page"""
 
@@ -928,6 +1014,13 @@ class GeneratedDatawatchClient(abc.ABC):
 
         return GetPreviewResponse().from_dict(self._call_datawatch(Method.GET, url))
 
+    def get_debug_and_metric_queries(self, *, metric_id: int = 0, timeout: int = 0) -> GetDebugQueriesResponse:
+        """Get queries for debug page"""
+        url = f"/api/v1/metrics/{metric_id}/debug/queries"
+        return GetDebugQueriesResponse().from_dict(
+            self._call_datawatch(Method.GET, url, timeout=timeout)
+        )
+
     def create_named_schedule(
             self, *, id: int = 0, name: str = "", cron: str = ""
     ) -> NamedSchedule:
@@ -976,9 +1069,19 @@ class GeneratedDatawatchClient(abc.ABC):
 
         url = f'/api/v1/schedules{encode_url_params(d=request.to_dict())}'
 
-        response = self._call_datawatch(Method.GET, url=url)
+        response = GetNamedSchedulesResponse().from_dict(self._call_datawatch(Method.GET, url=url))
+        schedules: List[NamedSchedule] = []
+        schedules.extend(response.named_schedules)
 
-        return GetNamedSchedulesResponse().from_dict(response)
+        while response.pagination_info.next_cursor:
+            request.page_cursor = response.pagination_info.next_cursor
+            url = f'/api/v1/schedules{encode_url_params(d=request.to_dict())}'
+            response = self._call_datawatch(Method.GET, url=url)
+            schedules.extend(
+                GetNamedSchedulesResponse().from_dict(response).named_schedules
+            )
+
+        return GetNamedSchedulesResponse(named_schedules=schedules)
 
     def create_data_node(self, *, node_type: DataNodeType, node_entity_id: int) -> DataNode:
         """Create Data Node for Lineage"""
@@ -1048,17 +1151,8 @@ class GeneratedDatawatchClient(abc.ABC):
         request.downstream_data_node_id = downstream_data_node_id
 
         url = '/api/v2/lineage/edges'
-
-        # If edge already exists, the call fails - catch and warn on this error instead
-        try:
-            response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
-            return LineageEdgeV2().from_dict(response)
-        except Exception as e:
-            exists_err = f"Edge from ({request.upstream_data_node_id}, {request.downstream_data_node_id}) already exists"
-            if exists_err not in str(e):
-                raise e
-            else:
-                log.info(exists_err)
+        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        return LineageEdgeV2().from_dict(response)
 
     def delete_lineage_relationship_for_node(self,
                                              *,
@@ -1068,6 +1162,40 @@ class GeneratedDatawatchClient(abc.ABC):
         log.info(f"Deleting lineage relationships for data node {data_node_id}")
         self._call_datawatch(Method.DELETE, url=url)
 
+    def create_lineage_node(self,
+                            node_name: str,
+                            node_type: DataNodeType,
+                            node_container_name: Optional[str] = None,
+                            icon_url: Optional[str] = None) -> LineageNodeV2:
+        url = '/api/v2/lineage/nodes'
+        request = CreateLineageNodeV2Request()
+        request.node_name = node_name
+        request.icon_url = icon_url
+
+        if node_container_name:
+            request.node_container_name = node_container_name
+
+        request_dict = request.to_dict()
+        request_dict["nodeType"] = node_type
+        response = self._call_datawatch(Method.POST, url=url, body=json.dumps(request_dict))
+        return LineageNodeV2().from_dict(response)
+
+    def delete_lineage_node(self, node_id: int):
+        url = f'/api/v2/lineage/nodes/{node_id}'
+        log.info(f"Deleting lineage node with ID: {node_id}")
+        return self._call_datawatch(Method.DELETE, url=url)
+
+    def get_lineage_graph_from_data_node(self,
+                                         *,
+                                         data_node_id: int,
+                                         depth: int = 1,
+                                         direction: LineageDirection = LineageDirection.ALL,
+                                         timeout: Optional[int] = None) -> TableLineageV2Response:
+        url = f"/api/v2/lineage/nodes/{data_node_id}/graph?depth={depth}&direction={direction}"
+        return TableLineageV2Response().from_dict(
+            self._call_datawatch(Method.GET, url=url, timeout=timeout)
+        )
+
     def execute_lineage_workflow(self, source_id: int):
         """Run lineage collection process for a source. For non-cloud warehouse vendors (Snowflake, Bigquery, Redshift)
         this will only create the necessary containment edges (source -> table -> column)"""
@@ -1099,11 +1227,10 @@ class GeneratedDatawatchClient(abc.ABC):
                                template: str,
                                return_type: Union[SimpleFieldType.BOOLEAN, SimpleFieldType.NUMERIC],
                                parameters: List[str],
-                               source: Source) -> CreateMetricTemplateResponse:
+                               source: Optional[Source] = None) -> CreateMetricTemplateResponse:
 
         metric_template = MetricTemplate()
         request = CreateMetricTemplateRequest()
-        warehouse = Warehouse(id=source.id, name=source.name, warehouse_vendor=source.database_type)
         params = []
 
         metric_template.id = id
@@ -1122,7 +1249,8 @@ class GeneratedDatawatchClient(abc.ABC):
             params.append(temp_param)
 
         metric_template.parameters = params
-        metric_template.source = warehouse
+        if source:
+            metric_template.source = Warehouse(id=source.id, name=source.name, warehouse_vendor=source.database_type)
 
         request.metric_template = metric_template
 
@@ -1173,10 +1301,11 @@ class GeneratedDatawatchClient(abc.ABC):
                              *,
                              name: str,
                              sql: str,
-                             warehouse_id: int):
+                             warehouse_id: int) -> VirtualTable:
         request = VirtualTableRequest(name=name, sql=sql, warehouse_id=warehouse_id)
         url = "/api/v1/virtual-tables"
-        self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        response = self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        return VirtualTable().from_dict(response)
 
     def update_virtual_table(self,
                              *,
@@ -1281,12 +1410,11 @@ class GeneratedDatawatchClient(abc.ABC):
             self._call_datawatch(Method.GET, url=url)
         ).integrations
 
-    def get_integration_entities(self, *, integration_id: int) -> List[TableauWorkbook]:
-        # TODO: Will require change once we onboard more integrations.
+    def get_integration_entities(self, *, integration_id: int) -> GetIntegrationEntitiesResponse:
         url = f"/api/v1/integrations/{integration_id}/entities"
         return GetIntegrationEntitiesResponse().from_dict(
             self._call_datawatch(Method.GET, url=url)
-        ).tableau_workbooks
+        )
 
     def get_advanced_configs(self) -> List[ConfigValue]:
         url = "/api/v1/configs/fetch"
@@ -1294,16 +1422,39 @@ class GeneratedDatawatchClient(abc.ABC):
             self._call_datawatch(Method.POST, url=url)
         ).advanced_configs
 
+    def update_advanced_configs(
+            self, *, key: str = "", value: Union[str, int, bool] = "", config_values: Optional[List[dict]] = None
+    ):
+        url = "/api/v1/configs/"
+        if not config_values:
+            config_values = {"key": key}
+            if isinstance(value, str):
+                config_values.update(
+                    {"stringValue": value, "type": ConfigValueType.VALUE_TYPE_STRING.name}
+                )
+            elif isinstance(value, bool):
+                config_values.update(
+                    {"booleanValue": value, "type": ConfigValueType.VALUE_TYPE_BOOL.name}
+                )
+            else:
+                config_values.update(
+                    {"numberValue": value, "type": ConfigValueType.VALUE_TYPE_INTEGER.name}
+                )
+
+        update_request = {"advancedConfigs": [config_values]}
+        self._call_datawatch(Method.PUT, url=url, body=json.dumps(update_request))
+
     def rebuild_schema(self, schema_id: int) -> WorkflowResponse:
         url = f"/api/v1/rebuilds/schemas/{schema_id}"
         return WorkflowResponse().from_dict(
             self._call_datawatch(method=Method.POST, url=url)
         )
 
-    def rebuild_source(self, source_id: int) -> WorkflowResponse:
+    def rebuild_source(self, source_id: int, priority: int = 1) -> WorkflowResponse:
         url = f"/api/v1/rebuilds/sources/{source_id}"
+        request = RebuildSourceRequest(priority=priority)
         return WorkflowResponse().from_dict(
-            self._call_datawatch(method=Method.POST, url=url)
+            self._call_datawatch(method=Method.POST, url=url, body=request.to_json())
         )
 
     def get_workflow_status(self, workflow_id: int) -> WorkflowStatusResponse:
@@ -1317,6 +1468,11 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.GET, url=url)
         return WorkspaceListResponse().from_dict(response)
 
+    def get_workspace(self, *, workspace_id: int) -> Workspace:
+        url = f'/api/v1/workspaces/{workspace_id}'
+        response = self._call_datawatch(Method.GET, url=url)
+        return Workspace().from_dict(response)
+
     def create_workspace(self, workspace_name: str) -> Workspace:
         url = '/api/v1/workspaces'
         request = CreateOrUpdateWorkspaceRequest(name=workspace_name)
@@ -1333,6 +1489,12 @@ class GeneratedDatawatchClient(abc.ABC):
         url = f'/api/v1/workspaces/{workspace_id}'
         return self._call_datawatch(Method.DELETE, url=url)
 
+    def get_workspace_accessors(self, *, workspace_id: int) -> GetWorkspaceAccessorsResponse:
+        url = f"/api/v1/workspaces/{workspace_id}/accessors"
+        return GetWorkspaceAccessorsResponse().from_dict(
+            self._call_datawatch(Method.GET, url=url)
+        )
+
     def get_users(self) -> GetUserListResponse:
         url = '/user'
         response = self._call_datawatch(Method.GET, url=url)
@@ -1382,13 +1544,30 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.GET, url=url)
         return RoleV2ListResponse().from_dict(response)
 
-    def edit_group_roles(self, group_id: int, role_ids: List[int], operation: RoleOperation) -> BulkResponse:
-        url = '/api/v1/groups/roles'
-        request = BulkChangeGroupRolesRequest(role_ids=role_ids,
-                                              group_id=group_id,
-                                              operation=operation)
+    def edit_group_roles(self, group: Group, role: RoleV2, workspace_ids: List[int], operation: RoleOperation) -> BulkChangeGroupGrantsResponse:
+        url = '/api/v1/groups/grants'
+        grants: List[Grant] = []
+
+        if operation == RoleOperation.ROLE_OPERATION_GRANT:
+            for wid in workspace_ids:
+                grants.append(Grant(
+                    role=role,
+                    group=IdAndDisplayName(id=group.id, display_name=group.name),
+                    workspace=IdAndDisplayName(id=wid)
+                ))
+        elif operation == RoleOperation.ROLE_OPERATION_REVOKE:
+            for grant in group.grants:
+                if grant.workspace.id in workspace_ids:
+                    grants.append(grant)
+        else:
+            raise Exception(f'Role operation {operation} not supported.')
+
+        request = BulkChangeGroupGrantsRequest(
+            operation=operation,
+            requests=grants
+        )
         response = self._call_datawatch(Method.PUT, url=url, body=request.to_json())
-        return BulkResponse().from_dict(response)
+        return BulkChangeGroupGrantsResponse().from_dict(response)
 
     def get_columns(self, table_ids: List[int] = [], column_ids: List[int] = []) -> GetColumnListResponse:
         url = '/api/v1/columns/fetch'
@@ -1467,7 +1646,24 @@ class GeneratedDatawatchClient(abc.ABC):
         response = self._call_datawatch(Method.GET, url=url)
         return CustomRuleInfo().from_dict(response)
 
-    def get_rules(self, warehouse_id: int) -> GetCustomRuleListResponse:
+    def get_rules_for_collection(self, collection_id: int) -> GetCustomRuleListResponse:
+        """Get all rules for a given collection."""
+        url = f'/api/v1/custom-rules/collection/{collection_id}'
+        response = self._call_datawatch(Method.GET, url=url)
+
+        rlist_current = GetCustomRuleListResponse().from_dict(response)
+        rlist_total = GetCustomRuleListResponse()
+        rlist_total.custom_rules.extend(rlist_current.custom_rules)
+
+        while rlist_current.pagination_info.next_cursor:
+            response.page_cursor = rlist_current.pagination_info.next_cursor
+            response = self._call_datawatch(Method.POST, url=url, body=response.to_json())
+            rlist_current = GetCustomRuleListResponse().from_dict(response)
+            rlist_total.custom_rules.extend(rlist_current.custom_rules)
+
+        return rlist_total
+
+    def get_rules_for_source(self, warehouse_id: int) -> GetCustomRuleListResponse:
         """Get all rules for a given warehouse"""
         url = f'/api/v1/custom-rules/warehouse/{warehouse_id}'
         response = self._call_datawatch(Method.GET, url=url)
@@ -1488,3 +1684,135 @@ class GeneratedDatawatchClient(abc.ABC):
         """Delete a custom rule."""
         url = f'/api/v1/custom-rules/{id}'
         return self._call_datawatch(Method.DELETE, url=url)
+
+    def get_custom_rule_list(self,
+                             *,
+                             search: str = "",
+                             page_size: int = 0,
+                             source_id: int = 0,
+                             schema_id: int = 0,
+                             table_id: int = 0,
+                             column_id: int = 0,
+                             collection_id: int = 0,
+                             join_id: int = 0):
+        url = "/api/v1/custom-rules/fetch"
+        request = GetCustomRuleListRequest(
+            search=search,
+            page_size=page_size,
+            source_id=source_id,
+            schema_id=schema_id,
+            table_id=table_id,
+            column_id=column_id,
+            collection_id=collection_id,
+            join_id=join_id
+        )
+
+        rlist_current = GetCustomRuleListResponse().from_dict(
+            self._call_datawatch(method=Method.POST, url=url, body=request.to_json())
+        )
+        rlist_total = GetCustomRuleListResponse()
+        rlist_total.custom_rules.extend(rlist_current.custom_rules)
+
+        while rlist_current.pagination_info.next_cursor:
+            rlist_current.page_cursor = rlist_current.pagination_info.next_cursor
+            response = self._call_datawatch(Method.POST, url=url, body=rlist_current.to_json())
+            rlist_current = GetCustomRuleListResponse().from_dict(response)
+            rlist_total.custom_rules.extend(rlist_current.custom_rules)
+
+        return rlist_total
+
+    def get_personal_api_keys(self) -> ListPersonalApiKeyResponse:
+        url = "/api/v1/personal-api-keys"
+        return ListPersonalApiKeyResponse().from_dict(
+            self._call_datawatch(Method.GET, url=url)
+        )
+
+    def create_personal_api_key(self, *, name: str, description: Optional[str] = "") -> CreatePersonalApiKeyResponse:
+        url = "/api/v1/personal-api-keys"
+        request = CreatePersonalApiKeyRequest()
+        request.name = name
+        request.description = description
+        return CreatePersonalApiKeyResponse().from_dict(
+            self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        )
+
+    def delete_personal_api_key(self, *, id: int):
+        url = f"/api/v1/personal-api-keys/{id}"
+        self._call_datawatch(Method.DELETE, url=url)
+
+    def get_agent_api_keys(self) -> ListAgentApiKeyResponse:
+        url = "/api/v1/agent-api-keys"
+        return ListAgentApiKeyResponse().from_dict(
+            self._call_datawatch(Method.GET, url=url)
+        )
+
+    def create_agent_api_key(self, *, name: str, description: Optional[str] = "") -> CreateAgentApiKeyResponse:
+        url = "/api/v1/agent-api-keys"
+        request = CreateAgentApiKeyRequest()
+        request.name = name
+        request.description = description
+        return CreateAgentApiKeyResponse().from_dict(
+            self._call_datawatch(Method.POST, url=url, body=request.to_json())
+        )
+
+    def delete_agent_api_key(self, *, id: int):
+        url = f"/api/v1/agent-api-keys/{id}"
+        self._call_datawatch(Method.DELETE, url=url)
+
+    def bulk_create_lineage_nodes(self, nodes: List[CreateLineageNodeV2Request]):
+        url = "/api/v2/lineage/nodes/bulk"
+        request = CreateLineageNodeV2BulkRequest()
+        request.nodes = nodes
+        return self._call_datawatch(Method.POST, url=url, body=request.to_json())
+
+    def bulk_create_lineage_edges(self, edges: List[CreateLineageEdgeV2Request]):
+        url = "/api/v2/lineage/edges/bulk"
+        request = CreateLineageEdgeV2BulkRequest()
+        request.edges = edges
+        return self._call_datawatch(Method.POST, url=url, body=request.to_json())
+
+    def get_bulk_metric_observed_column(
+            self,
+            *,
+            metric_ids: List[int] = [],
+            column_ids: List[int] = [],
+            table_ids: List[int] = [],
+            schema_ids: List[int] = [],
+            source_ids: List[int] = [],
+            is_for_custom_rule: bool = False,
+            remove_duplicate_metrics: bool = False
+    ) -> MetricObservedColumnListResponse:
+        url = "/api/v1/metric-observed-column/bulk-list"
+        request = GetMetricObservedColumnBulkRequest(
+            metric_ids=metric_ids,
+            column_ids=column_ids,
+            table_ids=table_ids,
+            schema_ids=schema_ids,
+            source_ids=source_ids,
+        ).to_dict()
+
+        request.update({"isForCustomRule": is_for_custom_rule, "removeDuplicateMetrics": remove_duplicate_metrics})
+        return MetricObservedColumnListResponse().from_dict(
+            self._call_datawatch(method=Method.POST, url=url, body=json.dumps(request))
+        )
+
+    def create_metric_observed_column(
+            self, *, column_id: int, metric_id: int, comments: Optional[str] = None
+    ) -> MetricObservedColumnResponse:
+        url = "/api/v1/metric-observed-column"
+        request = MetricObservedColumnRequest(column_id=column_id, metric_id=metric_id, comments=comments)
+        return MetricObservedColumnResponse().from_dict(
+            self._call_datawatch(method=Method.POST, url=url, body=request.to_json())
+        )
+
+    def get_metric_observed_column_for_metric(self, *, metric_id: int) -> MetricObservedColumnListResponse:
+        url = f"/api/v1/metric-observed-column/metric/{metric_id}"
+        return MetricObservedColumnListResponse().from_dict(
+            self._call_datawatch(method=Method.GET, url=url)
+        )
+
+    def get_metric_observed_column_for_column(self, *, column_id: int) -> MetricObservedColumnListResponse:
+        url = f"/api/v1/metric-observed-column/column/{column_id}"
+        return MetricObservedColumnListResponse().from_dict(
+            self._call_datawatch(method=Method.GET, url=url)
+        )
