@@ -3,8 +3,7 @@ import sys
 from pathlib import Path
 from typing import List, Dict, Tuple, Union, Optional
 
-from rich.prompt import Confirm
-from rich.text import Text
+import typer
 
 from bigeye_sdk.bigconfig_validation.big_config_reports import (
     raise_files_contain_error_exception,
@@ -30,7 +29,7 @@ from bigeye_sdk.exceptions.exceptions import (
     NoSourcesFoundException,
     FileNotFoundException,
 )
-from bigeye_sdk.functions.metric_functions import _is_table_level_metric
+from bigeye_sdk.functions.metric_functions import _is_table_level_metric, metric_template_has_name_without_id
 from bigeye_sdk.functions.search_and_match_functions import wildcard_search
 from bigeye_sdk.generated.com.bigeye.models.generated import (
     Source,
@@ -41,7 +40,7 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     MetricDefinition,
     FieldType,
     NamedSchedule,
-    User,
+    PredefinedMetricName,
 )
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.big_config import (
@@ -60,7 +59,7 @@ from bigeye_sdk.model.protobuf_message_facade import (
     SimpleCollection,
     SimpleMetricDefinition,
     SimpleSLA,
-    SimpleLookback,
+    SimpleLookback, SimpleTemplateMetric,
 )
 
 from bigeye_sdk.serializable import File, BIGCONFIG_FILE
@@ -88,7 +87,7 @@ def get_fq_name_from_cohort(cohort: CohortDefinition, source_name: str = None):
 
 
 def _find_bigconfig_files(
-    input_paths: List[str] = [], recursive: bool = False
+        input_paths: List[str] = [], recursive: bool = False
 ) -> List[BIGCONFIG_FILE]:
     """
     Finds bigconfig files either in specified input path or in working directory
@@ -147,14 +146,26 @@ class MetricSuiteController:
             v.id: v for k, v in self.sources_by_name_ix.items()
         }
         self.bigeye_users: Optional[Dict[str, int]] = None
+        self.templates_from_bigeye: Dict[str, int] = {}
 
-        self.table_level_metrics: List[SimplePredefinedMetricName] = [
+        self.table_level_metrics: List[SimplePredefinedMetricName] = self._get_table_level_metric_names()
+
+    def _get_table_level_metric_names(self):
+        # Filter out values that don't exist in the SDK, if generated.py has not been updated
+        raw_response = self.client.get_table_level_metrics_raw()
+        metric_names = []
+        for n in raw_response['metricNames']:
+            try:
+                metric_names.append(PredefinedMetricName.from_string(n))
+            except ValueError as e:
+                log.info(f"Error trying to create PredefinedMetric with {e}. Skipping to next")
+        return [
             SimplePredefinedMetricName.from_datawatch_object(pmn)
-            for pmn in self.client.get_table_level_metrics().metric_names
+            for pmn in metric_names
         ]
 
     def _upsert_collections(
-        self, bigconfig: BigConfig, overwrite: bool, apply=False
+            self, bigconfig: BigConfig, overwrite: bool, apply=False
     ) -> List[Union[SimpleSLA, SimpleCollection]]:
         """
         Currently operates as an overwrite of existing metrics.  This prevents altering in the front end.  Every time the
@@ -179,7 +190,7 @@ class MetricSuiteController:
                 )
                 merged_or_new = collection.merge_for_upsert(existing=existing)
                 deployment_collections.append(merged_or_new)
-                if apply:
+                if apply and existing != merged_or_new:
                     self.client.update_collection(
                         collection=merged_or_new.to_datawatch_object()
                     )
@@ -199,13 +210,13 @@ class MetricSuiteController:
         return deployment_collections
 
     def _get_matching_source_ids(
-        self, column_selector: ColumnSelector, source_include_pattern: str
+            self, column_selector: ColumnSelector, source_include_pattern: str
     ) -> List[int]:
         source_ids_to_include = [
             source.id
             for source_name, source in self.sources_by_name_ix.items()
             if source_name
-            in wildcard_search(
+               in wildcard_search(
                 search_string=source_include_pattern,
                 content=[source_name],
                 is_raw_regex=column_selector.is_raw_regex(),
@@ -213,8 +224,59 @@ class MetricSuiteController:
         ]
         return source_ids_to_include
 
+    def _update_metric_definition_for_deployment(self,
+                                                 m: SimpleMetricDefinition,
+                                                 named_schedules: List[NamedSchedule],
+                                                 default_lookback: SimpleLookback,
+                                                 deployment_collection: Union[SimpleSLA, SimpleCollection]
+                                                 ) -> Optional[SimpleMetricDefinition]:
+        if (
+                deployment_collection
+                and deployment_collection.id not in m.collection_ids
+        ):
+            m.collection_ids.append(deployment_collection.id)
+        if m.metric_schedule and m.metric_schedule.named_schedule:
+            try:
+                m.metric_schedule.named_schedule.id = [
+                    s.id
+                    for s in named_schedules
+                    if s.name == m.metric_schedule.named_schedule.name
+                ][0]
+            except IndexError:
+                log.warning(
+                    f"The schedule '{m.metric_schedule.named_schedule.name}' does not exist in workspace {self.client.config.workspace_id}"
+                )
+                log.warning("The metric will be created with no schedule.")
+                log.info(
+                    "Create the schedule via the CLI or the UI and the metric will be updated with the next deployment."
+                )
+                m.metric_schedule = None
+        if isinstance(m.metric_type, SimpleTemplateMetric) and metric_template_has_name_without_id(m.metric_type):
+            try:
+                if not self.templates_from_bigeye.get(m.metric_type.template_name.lower()):
+                    template = [
+                        t for t in self.client.get_all_metric_templates(search=m.metric_type.template_name)
+                        if t.name.lower() == m.metric_type.template_name.lower()
+                    ][0]
+                    self.templates_from_bigeye[template.name.lower()] = template.id
+
+                m.metric_type.template_id = self.templates_from_bigeye[m.metric_type.template_name.lower()]
+            except IndexError:
+                log.warning(
+                    f"No template found with name {m.metric_type.template_name}. Metric will not be created."
+                )
+                return None
+
+        m.user_favorites = self._generate_tags_for_favorites(m.user_favorites)
+        m.owner = self._get_owner_id(m.owner)
+
+        if not m.lookback:
+            m.lookback = default_lookback
+
+        return m
+
     def row_creation_times_to_cohort(
-        self, row_creation_times: RowCreationTimes
+            self, row_creation_times: RowCreationTimes
     ) -> Dict[int, List[CohortDefinition]]:
         r: Dict[int, List[CohortDefinition]] = {}
 
@@ -268,7 +330,7 @@ class MetricSuiteController:
         return r
 
     def table_deployment_to_row_creation_times_cohort(
-        self, td: TableDeployment
+            self, td: TableDeployment
     ) -> Dict[int, CohortDefinition]:
         r: Dict[int, CohortDefinition] = {}
 
@@ -284,7 +346,7 @@ class MetricSuiteController:
             source.id
             for source_name, source in self.sources_by_name_ix.items()
             if source_name
-            in wildcard_search(search_string=source_pattern, content=[source_name])
+               in wildcard_search(search_string=source_pattern, content=[source_name])
         ]
 
         if not matching_source_ids:
@@ -307,7 +369,7 @@ class MetricSuiteController:
         return r
 
     def _generate_tags_for_favorites(
-        self, user_favorites: List[str]
+            self, user_favorites: List[str]
     ) -> Optional[List[str]]:
         """
         Given a list of user emails, this will build a list of strings that include the corresponding Bigeye user id.
@@ -356,11 +418,11 @@ class MetricSuiteController:
         return str(owner_id)
 
     def tag_deployment_to_cohort_and_metric_def(
-        self,
-        tag_deployment: TagDeployment,
-        named_schedules: List[NamedSchedule],
-        default_lookback: SimpleLookback,
-        deployment_collection: Union[SimpleSLA, SimpleCollection] = None,
+            self,
+            tag_deployment: TagDeployment,
+            named_schedules: List[NamedSchedule],
+            default_lookback: SimpleLookback,
+            deployment_collection: Union[SimpleSLA, SimpleCollection] = None,
     ) -> Dict[int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TagDeployment object.   For table level metrics
@@ -381,35 +443,18 @@ class MetricSuiteController:
 
         for m in tag_deployment.metrics:
             """segregate table level metrics from column level and add slas"""
-            if (
-                deployment_collection
-                and deployment_collection.id not in m.collection_ids
-            ):
-                m.collection_ids.append(deployment_collection.id)
-            if m.metric_schedule and m.metric_schedule.named_schedule:
-                try:
-                    m.metric_schedule.named_schedule.id = [
-                        s.id
-                        for s in named_schedules
-                        if s.name == m.metric_schedule.named_schedule.name
-                    ][0]
-                except IndexError:
-                    log.warning(
-                        f"The schedule '{m.metric_schedule.named_schedule.name}' does not exist in workspace {self.client.config.workspace_id}"
-                    )
-                    log.warning("The metric will be created with the default schedule.")
-                    log.info(
-                        "Create the schedule via the CLI or the UI and the metric will be updated with the next deployment."
-                    )
-            m.user_favorites = self._generate_tags_for_favorites(m.user_favorites)
-            m.owner = self._get_owner_id(m.owner)
-
-            if not m.lookback:
-                m.lookback = default_lookback
+            m = self._update_metric_definition_for_deployment(
+                m=m,
+                named_schedules=named_schedules,
+                default_lookback=default_lookback,
+                deployment_collection=deployment_collection
+            )
+            if not m:
+                continue
 
             mdwo = m.to_datawatch_object()
             if _is_table_level_metric(
-                metric_type=m.metric_type, table_level_metrics=self.table_level_metrics
+                    metric_type=m.metric_type, table_level_metrics=self.table_level_metrics
             ):
                 mdwo.is_table_metric = True
                 table_metrics.append((m, mdwo))
@@ -471,7 +516,7 @@ class MetricSuiteController:
 
             if table_metrics:
                 if (column_pattern != "*" and cs.name) or (
-                    column_pattern != ".*" and cs.regex
+                        column_pattern != ".*" and cs.regex
                 ):
                     for smd, md in table_metrics:
                         errmsg = (
@@ -517,11 +562,11 @@ class MetricSuiteController:
         return cmds
 
     def table_deployment_to_cohort_and_metric_def(
-        self,
-        table_deployment: TableDeployment,
-        named_schedules: List[NamedSchedule],
-        default_lookback: SimpleLookback,
-        deployment_collection: Union[SimpleSLA, SimpleCollection] = None,
+            self,
+            table_deployment: TableDeployment,
+            named_schedules: List[NamedSchedule],
+            default_lookback: SimpleLookback,
+            deployment_collection: Union[SimpleSLA, SimpleCollection] = None,
     ) -> Dict[int, List[CohortAndMetricDefinition]]:
         """
         Builds a Cohort and MetricDefinition from a TableDeployment object
@@ -566,7 +611,7 @@ class MetricSuiteController:
         for m in table_deployment.table_metrics:
             "process table metrics and raise validation errors if column level metrics are defined."
             if not _is_table_level_metric(
-                metric_type=m.metric_type, table_level_metrics=self.table_level_metrics
+                    metric_type=m.metric_type, table_level_metrics=self.table_level_metrics
             ):
                 errmsg = (
                     f"Column level metrics cannot be applied at the table level.  "
@@ -581,34 +626,14 @@ class MetricSuiteController:
                     error_message=METRIC_APPLICATION_ERROR.format(errmsg=errmsg),
                 )
             else:
-                if (
-                    deployment_collection
-                    and deployment_collection.id not in m.collection_ids
-                ):
-                    m.collection_ids.append(deployment_collection.id)
-                if m.metric_schedule and m.metric_schedule.named_schedule:
-                    try:
-                        m.metric_schedule.named_schedule.id = [
-                            s.id
-                            for s in named_schedules
-                            if s.name == m.metric_schedule.named_schedule.name
-                        ][0]
-                    except IndexError:
-                        log.warning(
-                            f"The schedule '{m.metric_schedule.named_schedule.name}' does not exist in workspace {self.client.config.workspace_id}"
-                        )
-                        log.warning(
-                            "The metric will be created with the default schedule."
-                        )
-                        log.info(
-                            "Create the schedule via the CLI or the UI and the metric will be updated with the next deployment."
-                        )
-
-                if not m.lookback:
-                    m.lookback = default_lookback
-
-                m.user_favorites = self._generate_tags_for_favorites(m.user_favorites)
-                m.owner = self._get_owner_id(m.owner)
+                m = self._update_metric_definition_for_deployment(
+                    m=m,
+                    named_schedules=named_schedules,
+                    default_lookback=default_lookback,
+                    deployment_collection=deployment_collection
+                )
+                if not m:
+                    continue
 
                 mdwo = m.to_datawatch_object()
                 mdwo.is_table_metric = True
@@ -632,8 +657,8 @@ class MetricSuiteController:
 
             for m in c.metrics:
                 if _is_table_level_metric(
-                    metric_type=m.metric_type,
-                    table_level_metrics=self.table_level_metrics,
+                        metric_type=m.metric_type,
+                        table_level_metrics=self.table_level_metrics,
                 ):
                     errmsg = (
                         f"Table level metrics cannot be applied at the column level.  "
@@ -648,36 +673,14 @@ class MetricSuiteController:
                         error_message=METRIC_APPLICATION_ERROR.format(errmsg=errmsg),
                     )
                 else:
-                    if (
-                        deployment_collection
-                        and deployment_collection.id not in m.collection_ids
-                    ):
-                        m.collection_ids.append(deployment_collection.id)
-                    if m.metric_schedule and m.metric_schedule.named_schedule:
-                        try:
-                            m.metric_schedule.named_schedule.id = [
-                                s.id
-                                for s in named_schedules
-                                if s.name == m.metric_schedule.named_schedule.name
-                            ][0]
-                        except IndexError:
-                            log.warning(
-                                f"The schedule '{m.metric_schedule.named_schedule.name}' does not exist in workspace {self.client.config.workspace_id}"
-                            )
-                            log.warning(
-                                "The metric will be created with the default schedule."
-                            )
-                            log.info(
-                                "Create the schedule via the CLI or the UI and the metric will be updated with the next deployment."
-                            )
-
-                    if not m.lookback:
-                        m.lookback = default_lookback
-
-                    m.user_favorites = self._generate_tags_for_favorites(
-                        m.user_favorites
+                    m = self._update_metric_definition_for_deployment(
+                        m=m,
+                        named_schedules=named_schedules,
+                        default_lookback=default_lookback,
+                        deployment_collection=deployment_collection
                     )
-                    m.owner = self._get_owner_id(m.owner)
+                    if not m:
+                        continue
 
                     mdwo = m.to_datawatch_object()
                     col_metrics.append(mdwo)
@@ -693,9 +696,9 @@ class MetricSuiteController:
         return result
 
     def bigconfig_to_metric_suites(
-        self,
-        bigconfig: BigConfig,
-        deployment_collections: Dict[str, Union[SimpleSLA, SimpleCollection]],
+            self,
+            bigconfig: BigConfig,
+            deployment_collections: Dict[str, Union[SimpleSLA, SimpleCollection]],
     ) -> List[MetricSuite]:
         """
         Creates a MetricSuite for each source identified in a Bigconfig Table or Tag Deployment.
@@ -785,6 +788,7 @@ class MetricSuiteController:
                 source_id=source_id,
                 definitions=definitions,
                 auto_apply_on_indexing=bigconfig.auto_apply_on_indexing,
+                namespace=bigconfig.namespace
             )
             for source_id, definitions in cmds.items()
         }
@@ -803,17 +807,19 @@ class MetricSuiteController:
                     source_id=source_id,
                     row_creation_cohorts=rcts,
                     auto_apply_on_indexing=bigconfig.auto_apply_on_indexing,
+                    namespace=bigconfig.namespace
                 )
             )
 
         return metric_suites
 
     def execute_purge(
-        self,
-        purge_source_names: List[str] = None,
-        purge_all_sources: bool = False,
-        output_path: str = Path.cwd(),
-        apply: bool = False,
+            self,
+            purge_source_names: List[str] = None,
+            purge_all_sources: bool = False,
+            output_path: str = Path.cwd(),
+            apply: bool = False,
+            namespace: Optional[str] = None
     ):
         """
         Executes a purge of metrics deployed by MetricSuite on named sources or for all sources.
@@ -822,29 +828,31 @@ class MetricSuiteController:
             purge_all_sources: if true will purge all sources.
             output_path: path to dump the reports.  If no path is given the current working directory will be used.
             apply: If true then Big Config will be applied to the workspace.  If false then a plan will be generated.
+            namespace: The namespace of the bigconfig deployment
         """
         try:
             self.client.purge_metric_suites(
                 source_names=purge_source_names,
                 purge_all_sources=purge_all_sources,
                 apply=apply,
+                namespace=namespace
             )
             process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
             return process_reports(
-                output_path=output_path, strict_mode=False, process_stage=process_stage
+                output_path=output_path, strict_mode=False, process_stage=process_stage, namespace=namespace
             )
         except NoSourcesFoundException as e:
             sys.exit(e.message)
 
     def execute_bigconfig(
-        self,
-        input_path: List[str] = (Path.cwd()),
-        output_path: str = Path.cwd(),
-        apply: bool = False,
-        no_queue: bool = False,
-        recursive: bool = False,
-        strict_mode: bool = False,
-        auto_approve: bool = False,
+            self,
+            input_path: List[str] = (Path.cwd()),
+            output_path: str = Path.cwd(),
+            apply: bool = False,
+            recursive: bool = False,
+            strict_mode: bool = False,
+            auto_approve: bool = False,
+            namespace: Optional[str] = None
     ):
         """
         Executes an Apply or Plan for a Big Config.
@@ -852,10 +860,10 @@ class MetricSuiteController:
             input_path: path of source files.  If no path is given the current working directory will be used.
             output_path: path to dump the reports.  If no path is given the current working directory will be used.
             apply: If true, then Big Config will be applied to the workspace.  If false then a plan will be generated.
-            no_queue: If true, the bigconfig will be run without queuing
             recursive: If true, search for files recursively
             strict_mode: If true errors from the API raise an exception.
             auto_approve: If true, user will not be prompted to approve deployment
+            namespace: The namespace of the bigconfig deployment
         Returns: None
 
         """
@@ -863,7 +871,7 @@ class MetricSuiteController:
             input_paths=input_path, recursive=recursive
         )
 
-        bigconfig: BigConfig = combine_bigconfigs(files)
+        bigconfig: BigConfig = combine_bigconfigs(bigconfigs=files, namespace=namespace)
 
         if get_validation_error_cnt():
             """Processing validation errors if any exist and throw exception."""
@@ -909,12 +917,11 @@ class MetricSuiteController:
                 output_path=output_path,
                 strict_mode=strict_mode,
                 apply=False,
-                no_queue=no_queue,
+                namespace=bigconfig.namespace
             )
-            approved = Confirm.ask(
-                Text(
-                    f"Please review expected results. Are you sure you want to continue?",
-                    style="bold red",
+            approved = typer.confirm(
+                typer.style(
+                    f"Please review expected results. Are you sure you want to continue?", fg="red", bold=True
                 )
             )
             if approved:
@@ -923,27 +930,27 @@ class MetricSuiteController:
                     output_path=output_path,
                     strict_mode=strict_mode,
                     apply=apply,
-                    no_queue=no_queue,
+                    namespace=bigconfig.namespace
                 )
         else:
-            self._process_metric_suites(
+            return self._process_metric_suites(
                 metric_suites=metric_suites,
                 output_path=output_path,
                 strict_mode=strict_mode,
                 apply=apply,
-                no_queue=no_queue,
+                namespace=bigconfig.namespace
             )
 
     def _process_metric_suites(
-        self,
-        metric_suites: List[MetricSuite],
-        output_path: str = Path.cwd(),
-        strict_mode: bool = False,
-        apply: bool = False,
-        no_queue: bool = False,
+            self,
+            metric_suites: List[MetricSuite],
+            output_path: str = Path.cwd(),
+            strict_mode: bool = False,
+            apply: bool = False,
+            namespace: Optional[str] = None
     ):
         """Send metric suites to correct endpoints and process reports."""
-        run_metric_on_save = None
+        run_metric_on_save = False
 
         # if applying, clear out any plan reports that were needed for approvals.
         if apply:
@@ -966,15 +973,7 @@ class MetricSuiteController:
         process_stage = ProcessStage.APPLY if apply else ProcessStage.PLAN
 
         for metric_suite in metric_suites:
-            if no_queue:
-                response = self.client.post_metric_suite(
-                    metric_suite=metric_suite, apply=apply
-                )
-            else:
-                response = self.client.post_metric_suite_queue(
-                    metric_suite=metric_suite, apply=apply
-                )
-
+            response = self.client.post_bigconfig(metric_suite=metric_suite, apply=apply)
             MetricSuiteReport.from_datawatch_object(
                 response,
                 source_name=self.sources_by_id_ix[metric_suite.source_id].name,
@@ -987,10 +986,11 @@ class MetricSuiteController:
             strict_mode=strict_mode,
             process_stage=process_stage,
             run_metric_on_save=run_metric_on_save,
+            namespace=namespace
         )
 
 
-def combine_bigconfigs(bigconfigs: List[BigConfig]) -> BigConfig:
+def combine_bigconfigs(bigconfigs: List[BigConfig], namespace: Optional[str] = None) -> BigConfig:
     to_return: BigConfig = BigConfig(
         type="BIGCONFIG_FILE",
         tag_definitions=[],
@@ -999,21 +999,42 @@ def combine_bigconfigs(bigconfigs: List[BigConfig]) -> BigConfig:
         tag_deployments=[],
         table_deployments=[],
     )
-    auto_apply: bool = bigconfigs[-1].auto_apply_on_indexing
+    from statistics import mode
+    auto_apply_to_check: bool = mode([bc.auto_apply_on_indexing for bc in bigconfigs])
+    namespace_to_check: Optional[str] = mode([bc.namespace for bc in bigconfigs])
+    namespaces_match = True
+
+    if namespace:
+        for bc in bigconfigs:
+            if bc.namespace != namespace and bc.namespace is not None:
+                namespaces_match = False
+        if not namespaces_match:
+            raise BigConfigValidationException(
+                f"The namespace from the CLI does not match the namespace defined in the file(s). "
+                f"Remove the argument from the command line or the namespace parameter from the file(s)."
+            )
+
+        if not namespace_to_check:
+            namespace_to_check = namespace
+
     for bc in bigconfigs:
-        if auto_apply != bc.auto_apply_on_indexing:
-            "registering validation errors when auto_apply doesn't match across bigconfigs"
-            errlns = [
-                f"auto_apply_on_indexing: {auto_apply}\n",
-                f"auto_apply_on_indexing: {bc.auto_apply_on_indexing}\n",
-            ]
+        if auto_apply_to_check != bc.auto_apply_on_indexing:
             BigConfig.register_validation_error(
-                error_lines=errlns,
+                error_lines=[f"auto_apply_on_indexing: {bc.auto_apply_on_indexing}".lower()],
                 error_message=MISMATCHED_ATTRIBUTE_ACROSS_MULTIPLE_FILES.format(
                     attribute="auto_apply_on_indexing"
                 ),
             )
-        to_return.auto_apply_on_indexing = auto_apply
+
+        if namespace_to_check != bc.namespace:
+            if bc.namespace is not None or namespace is None:
+                BigConfig.register_validation_error(
+                    error_lines=[f"namespace: {bc.namespace}"],
+                    error_message=MISMATCHED_ATTRIBUTE_ACROSS_MULTIPLE_FILES.format(
+                        attribute="namespace"
+                    ),
+                )
+        to_return.auto_apply_on_indexing = auto_apply_to_check
         to_return.tag_definitions.extend(bc.tag_definitions)
         if bc.row_creation_times:
             rct = bc.row_creation_times
@@ -1025,6 +1046,8 @@ def combine_bigconfigs(bigconfigs: List[BigConfig]) -> BigConfig:
         to_return.tag_deployments.extend(bc.tag_deployments)
         to_return.table_deployments.extend(bc.table_deployments)
 
+    if namespace_to_check:
+        to_return.namespace = namespace_to_check
     to_return.build_tag_ix(to_return.tag_definitions)
     to_return.build_saved_metric_ix(to_return.saved_metric_definitions)
 
