@@ -20,7 +20,7 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     SimpleBound,
     SimpleBoundType,
     SlackChannelInfo,
-    MetricRunStatus,
+    MetricRunStatus, MetricInfoList,
 )
 from bigeye_sdk.log import get_logger
 from bigeye_sdk.model.protobuf_enum_facade import (
@@ -31,13 +31,14 @@ from bigeye_sdk.model.protobuf_enum_facade import (
 from bigeye_sdk.model.protobuf_message_facade import (
     SimplePredefinedMetric,
     SimpleMetricType,
-    SimpleThreshold,
+    SimpleThreshold, SimpleTemplateMetric,
 )
 
 log = get_logger(__file__)
 
 
-def _is_table_level_metric(metric_type: SimpleMetricType, table_level_metrics: List[SimplePredefinedMetricName]) -> bool:
+def _is_table_level_metric(metric_type: SimpleMetricType,
+                           table_level_metrics: List[SimplePredefinedMetricName]) -> bool:
     if not isinstance(metric_type, SimplePredefinedMetric):
         return False
     if metric_type.predefined_metric in table_level_metrics:
@@ -45,6 +46,7 @@ def _is_table_level_metric(metric_type: SimpleMetricType, table_level_metrics: L
 
     return False
 
+
 def filter_metrics_by_table_ids(metrics: List[dict], table_ids: List[int]) -> List[dict]:
     """
     Deprecated.  Previously used to filter raw HTTP responses.
@@ -374,7 +376,7 @@ def get_time_interval_for_delay_string(delay_at_update: str, metric_type: str, u
     >>> get_time_interval_for_delay_string("1 day", "HOURS_SINCE_MAX_DATE", "10 14 * * 1")
     {'intervalValue': 38, 'intervalType': 'HOURS_TIME_INTERVAL_TYPE'}
     """
-    split_input = delay_at_update.split(" ",1)
+    split_input = delay_at_update.split(" ", 1)
     interval_value = int(split_input[0])
     interval_type = get_proto_interval_type(split_input[1])
     if metric_type == "HOURS_SINCE_MAX_DATE":
@@ -619,16 +621,41 @@ def get_metric_status(metric_info: MetricInfo) -> MetricStatus:
 
 
 def get_metric_lookback_window(metric_config: MetricConfiguration) -> str:
+    lookback_days = metric_config.lookback.interval_value
+    if lookback_days == -1:
+        lookback_days_string = "Inferred"
+    else:
+        lookback_days_string = f"{metric_config.lookback.interval_value} Days"
+
     if metric_config.lookback_type == LookbackType.METRIC_TIME_LOOKBACK_TYPE:
-        return f"Data Time Window ({metric_config.lookback.interval_value} Days)"
+        return f"Data Time Window ({lookback_days_string})"
     elif metric_config.lookback_type == LookbackType.DATA_TIME_LOOKBACK_TYPE:
-        return f"Data Time ({metric_config.lookback.interval_value} Days)"
+        return f"Data Time ({lookback_days_string})"
     elif metric_config.lookback_type == LookbackType.CLOCK_TIME_LOOKBACK_TYPE:
-        return f"Clock Time ({metric_config.lookback.interval_value} Days)"
+        return f"Clock Time ({lookback_days_string})"
     elif metric_config.lookback_type == LookbackType.UNDEFINED_LOOKBACK_TYPE:
         return "Full Scan"
 
 
+def create_metric_info_list(metric_infos: dict) -> MetricInfoList:
+    mil_current = MetricInfoList()
+    infos: List[MetricInfo] = []
+
+    for metric_info in metric_infos["metrics"]:
+        try:
+            infos.append(MetricInfo().from_dict(metric_info))
+        except ValueError:
+            pass
+    mil_current.metrics = infos
+    mil_current.pagination_info = metric_infos["paginationInfo"]
+
+    return mil_current
+
+
+def metric_template_has_name_without_id(metric_template: SimpleTemplateMetric) -> bool:
+    return metric_template.template_name and not metric_template.template_id
+
+
 if __name__ == "__main__":
     import doctest
 
