@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import abc
 
-from pydantic.dataclasses import dataclass
+from pydantic.v1.dataclasses import dataclass
 
 # create logger
 from bigeye_sdk import DatawatchObject
