@@ -331,6 +331,7 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
     are applied -- and validated -- during the __post_init__ phase of instantiating a Bigconfig.
     """
     auto_apply_on_indexing: bool = False
+    namespace: Optional[str] = None
     tag_definitions: Optional[List[TagDefinition]] = Field(
         default_factory=lambda: [])  # only one because we must consolidate if creating Bigconfig from multiple files.
     row_creation_times: Optional[RowCreationTimes] = RowCreationTimes()
@@ -698,8 +699,6 @@ class BigConfig(BigConfigFile, type='BIGCONFIG_FILE'):
             for td in tag_deployments:
                 for metric in td.metrics:
                     cleaned_metric = _remove_default_values(metric)
-                    # assume conditions are always equal as we arbitrarily update conditions for 1+ matching types
-                    saved_metric.conditions = cleaned_metric.conditions
                     if cleaned_metric == saved_metric:
                         column_selectors.extend(td.column_selectors)
 
