@@ -31,7 +31,7 @@ class SimpleDbtColumn(BaseModel):
     @root_validator(pre=True)
     def validate_tests(cls, values):
         tests = []
-        if values["tests"]:
+        if values.get("tests"):
             for test in values["tests"]:
                 params = [SimpleMetricParameter(key="arg1", column_name=values["name"])]
                 if isinstance(test, dict) and list(test.keys())[0] == "accepted_values":
