@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import enum
 from enum import Enum
 from itertools import chain
 from typing import List
@@ -23,12 +24,12 @@ from bigeye_sdk.generated.com.bigeye.models.generated import (
     WorkflowProcessingStatus,
     TableSortField,
     SortDirection,
-    SchemaChangeOperation,
+    SchemaChangeOperation, IssuePriority, IssueSortField, IssueStatus, RelationshipType, IntegrationPartner,
 )
 
 
 def _remove_protobuf_type(
-    protobuf_enum_cls: type(betterproto.Enum), member_name: str
+        protobuf_enum_cls: type(betterproto.Enum), member_name: str
 ) -> str:
     """
     Processes an enum type prefix or postfix out of an enum member name.
@@ -103,7 +104,7 @@ def datawatch_enum_facade(cls):
     return cls
 
 
-class MetricStatus(YamlStrEnum):
+class MetricStatus(str, enum.Enum):
     HEALTHY = "HEALTHY"  # Query by this status will contain healthy metrics
     ALERTING = "ALERTING"  # Query by this status will contain alerting metrics
     FAILED = "FAILED"
@@ -111,7 +112,7 @@ class MetricStatus(YamlStrEnum):
 
 
 # TODO deprecate
-class SimpleMetricCategory(YamlStrEnum):
+class SimpleMetricCategory(str, enum.Enum):
     PREDEFINED = "PREDEFINED"
     TEMPLATE = "TEMPLATE"
 
@@ -145,7 +146,7 @@ class SimpleMetricCategory(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimplePredefinedMetricName(YamlStrEnum):
+class SimplePredefinedMetricName(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to PredefinedMetricName."""
 
     _ignore_ = "member cls"
@@ -158,7 +159,7 @@ class SimplePredefinedMetricName(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleAutothresholdSensitivity(YamlStrEnum):
+class SimpleAutothresholdSensitivity(str, enum.Enum):
     """Programmatically provides a yaml serializable autothreshold from string value to AutothresholdSensitivity"""
 
     _ignore_ = "member cls"
@@ -171,7 +172,7 @@ class SimpleAutothresholdSensitivity(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleFieldType(YamlStrEnum):
+class SimpleFieldType(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to FieldType"""
 
     _ignore_ = "member cls"
@@ -184,7 +185,7 @@ class SimpleFieldType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleTimeIntervalType(YamlStrEnum):
+class SimpleTimeIntervalType(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to TimeIntervalType"""
 
     _ignore_ = "member cls"
@@ -197,7 +198,7 @@ class SimpleTimeIntervalType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleLookbackType(YamlStrEnum):
+class SimpleLookbackType(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to LookbackType"""
 
     _ignore_ = "member cls"
@@ -210,7 +211,7 @@ class SimpleLookbackType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleAggregationType(YamlStrEnum):
+class SimpleAggregationType(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to AggregationType"""
 
     _ignore_ = "member cls"
@@ -223,8 +224,8 @@ class SimpleAggregationType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleDataNodeType(YamlStrEnum):
-    """Programmatically provides a yaml serializable mapping from string value to FieldType"""
+class SimpleDataNodeType(str, enum.Enum):
+    """Programmatically provides a yaml serializable mapping from string value to DataNodeType"""
 
     _ignore_ = "member cls"
     __protobuf_enum_cls__ = DataNodeType
@@ -236,7 +237,7 @@ class SimpleDataNodeType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleMetricTemplateParameterType(YamlStrEnum):
+class SimpleMetricTemplateParameterType(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to MetricTemplateParameterType"""
 
     _ignore_ = "member cls"
@@ -250,7 +251,7 @@ class SimpleMetricTemplateParameterType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleCatalogEntityType(YamlStrEnum):
+class SimpleCatalogEntityType(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to MetricTemplateParameterType"""
 
     _ignore_ = "member cls"
@@ -263,7 +264,7 @@ class SimpleCatalogEntityType(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleWorkflowProcessingStatus(YamlStrEnum):
+class SimpleWorkflowProcessingStatus(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to WorkflowProcessingStatus"""
 
     _ignore_ = "member cls"
@@ -276,7 +277,7 @@ class SimpleWorkflowProcessingStatus(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleTableSortField(YamlStrEnum):
+class SimpleTableSortField(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to TableSortField"""
 
     _ignore_ = "member cls"
@@ -289,7 +290,20 @@ class SimpleTableSortField(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleSortDirection(YamlStrEnum):
+class SimpleIssueSortField(str, enum.Enum):
+    """Programmatically provides a yaml serializable mapping from string value to IssueSortField"""
+
+    _ignore_ = "member cls"
+    __protobuf_enum_cls__ = IssueSortField
+    cls = vars()
+    for member in list(__protobuf_enum_cls__):
+        cls[_remove_protobuf_type(__protobuf_enum_cls__, member.name)] = (
+            _remove_protobuf_type(__protobuf_enum_cls__, member.name)
+        )
+
+
+@datawatch_enum_facade
+class SimpleSortDirection(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to SortDirection"""
 
     _ignore_ = "member cls"
@@ -302,7 +316,7 @@ class SimpleSortDirection(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleSchemaChangeOperation(YamlStrEnum):
+class SimpleSchemaChangeOperation(str, enum.Enum):
     """Programmatically provides a yaml serializable mapping from string value to SchemaChangeOperation"""
 
     _ignore_ = "member cls"
@@ -316,13 +330,53 @@ class SimpleSchemaChangeOperation(YamlStrEnum):
 
 
 @datawatch_enum_facade
-class SimpleMetricTemplateFieldType(YamlStrEnum):
+class SimpleIssuePriority(str, enum.Enum):
+    """Programmatically provides a yaml serializable mapping from string value to IssuePriority"""
+
+    _ignore_ = "member cls"
+    __protobuf_enum_cls__ = IssuePriority
+    cls = vars()
+    for member in list(__protobuf_enum_cls__):
+        if "UNSPECIFIED" not in member.name:
+            cls[_remove_protobuf_type(__protobuf_enum_cls__, member.name)] = (
+                _remove_protobuf_type(__protobuf_enum_cls__, member.name)
+            )
+
+
+@datawatch_enum_facade
+class SimpleIssueStatus(str, enum.Enum):
+    """Programmatically provides a yaml serializable mapping from string value to IssueStatus"""
+
+    _ignore_ = "member cls"
+    __protobuf_enum_cls__ = IssueStatus
+    cls = vars()
+    for member in list(__protobuf_enum_cls__):
+        cls[_remove_protobuf_type(__protobuf_enum_cls__, member.name)] = (
+            _remove_protobuf_type(__protobuf_enum_cls__, member.name)
+        )
+
+
+@datawatch_enum_facade
+class SimpleIntegrationPartner(str, enum.Enum):
+    """Programmatically provides a yaml serializable mapping from string value to IntegrationPartner"""
+
+    _ignore_ = "member cls"
+    __protobuf_enum_cls__ = IntegrationPartner
+    cls = vars()
+    for member in list(__protobuf_enum_cls__):
+        cls[_remove_protobuf_type(__protobuf_enum_cls__, member.name)] = (
+            _remove_protobuf_type(__protobuf_enum_cls__, member.name)
+        )
+
+
+@datawatch_enum_facade
+class SimpleMetricTemplateFieldType(str, enum.Enum):
     BOOLEAN = SimpleFieldType.BOOLEAN
     NUMERIC = SimpleFieldType.NUMERIC
 
 
 @datawatch_enum_facade
-class SimpleDbtTestToMetricType(YamlStrEnum):
+class SimpleDbtTestToMetricType(str, enum.Enum):
     not_null = SimplePredefinedMetricName.COUNT_NULL
     unique = SimplePredefinedMetricName.COUNT_DUPLICATES
     accepted_values = SimplePredefinedMetricName.PERCENT_VALUE_IN_LIST
