@@ -37,13 +37,15 @@ freshness_metrics = [
     SimplePredefinedMetricName.HOURS_SINCE_LAST_LOAD,
     SimplePredefinedMetricName.HOURS_SINCE_MAX_DATE,
     SimplePredefinedMetricName.HOURS_SINCE_MAX_TIMESTAMP,
-    SimplePredefinedMetricName.FRESHNESS
+    SimplePredefinedMetricName.FRESHNESS,
+    SimplePredefinedMetricName.FRESHNESS_DATA
 ]
 
 volume_metrics = [
     SimplePredefinedMetricName.VOLUME,
     SimplePredefinedMetricName.ROWS_INSERTED,
-    SimplePredefinedMetricName.COUNT_ROWS
+    SimplePredefinedMetricName.COUNT_ROWS,
+    SimplePredefinedMetricName.VOLUME_DATA
 ]
 
 pipeline_reliability_metrics = [SimplePredefinedMetricName.COUNT_READ_QUERIES] + freshness_metrics + volume_metrics
@@ -61,7 +63,9 @@ completeness_metrics = [
 
 uniqueness_metrics = [
     SimplePredefinedMetricName.COUNT_DISTINCT,
-    SimplePredefinedMetricName.COUNT_DUPLICATES
+    SimplePredefinedMetricName.COUNT_DUPLICATES,
+    SimplePredefinedMetricName.PERCENT_DISTINCT,
+    SimplePredefinedMetricName.PERCENT_DUPLICATES
 ]
 
 distribution_metrics = [
@@ -122,7 +126,8 @@ validity_metrics = [
     SimplePredefinedMetricName.PERCENT_TIMESTAMP_STRING,
     SimplePredefinedMetricName.COUNT_USA_STATE_CODE,
     SimplePredefinedMetricName.PERCENT_USA_STATE_CODE,
-    SimplePredefinedMetricName.PERCENT_VALUE_IN_LIST
+    SimplePredefinedMetricName.PERCENT_VALUE_IN_LIST,
+    SimplePredefinedMetricName.COUNT_VALUE_IN_LIST
 ]
 
 
@@ -137,11 +142,7 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
 
     @classmethod
     def get_freshness_metric_types(cls) -> List[SimplePredefinedMetric]:
-        freshness_metric_names = [SimplePredefinedMetricName.HOURS_SINCE_LAST_LOAD,
-                                  SimplePredefinedMetricName.HOURS_SINCE_MAX_DATE,
-                                  SimplePredefinedMetricName.HOURS_SINCE_MAX_TIMESTAMP,
-                                  SimplePredefinedMetricName.FRESHNESS]
-        return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in freshness_metric_names]
+        return cls.__get_simple_predefined_metrics(metric_names=freshness_metrics)
 
     @classmethod
     def get_volume_metric_types(cls) -> List[SimplePredefinedMetric]:
@@ -203,7 +204,9 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
     def is_freshness_volume(cls, predefined_metric):
         return predefined_metric in [
             SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.FRESHNESS),
-            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME)
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.FRESHNESS_DATA),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME_DATA)
         ]
 
     @classmethod
@@ -211,6 +214,8 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
         return predefined_metric in [
             SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.FRESHNESS),
             SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.FRESHNESS_DATA),
+            SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.VOLUME_DATA),
             SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.COUNT_ROWS),
             SimplePredefinedMetric(type="PREDEFINED", predefined_metric=SimplePredefinedMetricName.COUNT_READ_QUERIES),
         ]
@@ -221,7 +226,10 @@ class SimpleMetricType(PydanticSubtypeSerializable, DatawatchFacade, ABC):
                             SimplePredefinedMetricName.FRESHNESS,
                             SimplePredefinedMetricName.VOLUME,
                             SimplePredefinedMetricName.ROWS_INSERTED,
-                            SimplePredefinedMetricName.COUNT_READ_QUERIES]
+                            SimplePredefinedMetricName.COUNT_READ_QUERIES,
+                            SimplePredefinedMetricName.FRESHNESS_DATA,
+                            SimplePredefinedMetricName.VOLUME_DATA
+                            ]
         return [SimplePredefinedMetric(type="PREDEFINED", predefined_metric=i) for i in metadata_metrics]
 
     @classmethod
@@ -284,7 +292,7 @@ class SimplePredefinedMetric(SimpleMetricType, type='PREDEFINED'):
 
 class SimpleTemplateMetric(SimpleMetricType, type='TEMPLATE'):
     type: str = 'TEMPLATE'
-    template_id: int
+    template_id: int = 0
     aggregation_type: SimpleAggregationType
     template_name: Optional[str] = None
 
@@ -650,8 +658,8 @@ class SimpleStdDevThreshold(SimpleThreshold, type='STDDEV'):
 
 class SimpleNoneThreshold(SimpleThreshold, type='NONE'):
 
-    def to_datawatch_object(self, **kwargs) -> Threshold:
-        return Threshold(none_threshold=NoneThreshold())
+    def to_datawatch_object(self, **kwargs) -> List[Threshold]:
+        return [Threshold(none_threshold=NoneThreshold())]
 
 
 SNC = TypeVar('SNC', bound='SimpleNotificationChannel')
@@ -745,10 +753,17 @@ class SimpleMetricParameter(YamlModelWithValidatorContext, DatawatchFacade):
                                      number_value=obj.number_value)
 
     def to_datawatch_object(self, **kwargs) -> MetricParameter:
-        return MetricParameter(key=self.key,
-                               string_value=self.string_value,
-                               column_name=self.column_name,
-                               number_value=self.number_value)
+        # This is required to handle 0 value defaults that get removed when calling to_json
+        if self.number_value == 0.0:
+            return MetricParameter(key=self.key,
+                                   string_value='0.0',
+                                   column_name=self.column_name,
+                                   number_value=self.number_value)
+        else:
+            return MetricParameter(key=self.key,
+                                   string_value=self.string_value,
+                                   column_name=self.column_name,
+                                   number_value=self.number_value)
 
 
 @deprecated('SimpleSLA is deprecated and will be removed in future versions. Use SimpleCollection instead.')
@@ -851,6 +866,16 @@ class SimpleCollection(YamlModelWithValidatorContext, DatawatchFacade):
     notification_channels: List[SimpleNotificationChannel] = Field(default_factory=lambda: [])
     muted_until_timestamp: int = 0
 
+    def __eq__(self, other: SimpleCollection):
+        if self.name == other.name \
+                and self.description == other.description \
+                and self.metric_ids == other.metric_ids \
+                and self.notification_channels == other.notification_channels \
+                and self.muted_until_timestamp == other.muted_until_timestamp:
+            return True
+        else:
+            return False
+
     @root_validator(pre=True)
     def must_be_list(cls, values):
         must_be_list_validator(clazz=SimpleCollection, attribute_name='metric_ids', values=values)
@@ -1197,7 +1222,8 @@ class SimpleMetricDefinition(YamlModelWithValidatorContext, DatawatchFacade):
         builder.conditions = obj.filters
         builder.group_by = obj.group_bys
 
-        builder.threshold = SimpleThreshold.from_datawatch_object(obj.thresholds)
+        if obj.thresholds:
+            builder.threshold = SimpleThreshold.from_datawatch_object(obj.thresholds)
 
         builder.notification_channels = [SimpleNotificationChannel.from_datawatch_object(nc)
                                          for nc in obj.notification_channels]
